package weather

import (
	"time"

	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/ambient/config"
)

// Stable intermediate struct containing superset of fields
// between AWN and Wunderground style updates from Ambient devices
type WeatherUpdate struct {
	DateUTC         *time.Time
	StationConfig   *config.WeatherStation
	StationID       *string
	StationType     *string
	TempOutdoorF    *float64
	TempIndoorF     *float64
	HumidityOudoor  *int
	HumidityIndoor  *int
	WindSpeedMPH    *float64
	WindGustMPH     *float64
	MaxDailyGust    *float64
	WindDir         *int
	WindDirAvg10m   *int
	UV              *int
	SolarRadiation  *float64
	HourlyRainIn    *float64
	EventRainIn     *float64
	DailyRainIn     *float64
	WeeklyRainIn    *float64
	MonthlyRainIn   *float64
	YearlyRainIn    *float64
	TotalRainIn     *float64
	Batteries       []BatteryStatus
	BaromRelativeIn *float64
	BaromAbsoluteIn *float64
	// These fields may be calculated
	// if not otherwise set
	DewPointF  *float64
	WindChillF *float64
	// Extra Temp+Humidity Sensors
	TempHumiditySensors []*TempHumiditySensor
}

type TempHumiditySensor struct {
	Name     string
	TempF    *float64
	Humidity *int
}

type BatteryStatus struct {
	Component string
	Status    *int
}

// CHORE: Maintain this, used to check against
// keep and drop lists
// TODO: Use refelct/ast to generate code
const (
	FieldDateUTC         = "DateUTC"
	FieldStationType     = "StationType"
	FieldTempOutdoorF    = "TempOutdoorF"
	FieldTempIndoorF     = "TempIndoorF"
	FieldHumidityOudoor  = "HumidityOudoor"
	FieldHumidityIndoor  = "HumidityIndoor"
	FieldWindSpeedMPH    = "WindSpeedMPH"
	FieldWindGustMPH     = "WindGustMPH"
	FieldMaxDailyGust    = "MaxDailyGust"
	FieldWindDir         = "WindDir"
	FieldWindDirAvg10m   = "WindDirAvg10m"
	FieldUV              = "UV"
	FieldSolarRadiation  = "SolarRadiation"
	FieldHourlyRainIn    = "HourlyRainIn"
	FieldEventRainIn     = "EventRainIn"
	FieldDailyRainIn     = "DailyRainIn"
	FieldWeeklyRainIn    = "WeeklyRainIn"
	FieldMonthlyRainIn   = "MonthlyRainIn"
	FieldYearlyRainIn    = "YearlyRainIn"
	FieldTotalRainIn     = "TotalRainIn"
	FieldBatteries       = "Batteries"
	FieldBaromRelativeIn = "BaromRelativeIn"
	FieldBaromAbsoluteIn = "BaromAbsoluteIn"
	FieldDewPointF       = "DewPointF"
	FieldWindChillF      = "WindChillF"
	FieldSensorTempF     = "SensorTempF"
	FieldSensorHumidity  = "SensorHumidity"
)

func (u *WeatherUpdate) GetStationName() string {
	if u.StationConfig != nil {
		return u.StationConfig.Name
	}
	return ""
}
